#!/bin/bash
source autosumo_parameters.sh


# 1. Download OpenStreetMap map
#mapa=wget https://www.openstreetmap.org/export#map=16/40.4267/-3.6816

mkdir -p ${folder}
cd ${folder}
ln -sf ../../$map .    # Link to where is the map (downloaded manually at the moment)
rm ${conf_file}

redmodify=1
dfrouter=1
echo "redmodify= "${redmodify}
echo "dfrouter= "${dfrouter}
if [ ${redmodify} -eq 0 ]; then

	# 2. Create SUMO network from .osm map
	#netconvert --osm-files ${map} --output-file ${network} --geometry.remove --roundabouts.guess --ramps.guess --junctions.join --tls.guess-signals --tls.discard-simple --tls.join
	echo "Running netconvert..."
	#netconvert --osm-files ${map} --output-file ${network} --osm.stop-output.length 20 --ptstop-output ${ptstops_out} --ptline-output ${ptlines_out} --railway.topology.output ${railway} --osm.elevation --geometry.remove --roundabouts.guess --ramps.guess --junctions.join --tls.guess-signals --tls.discard-simple --tls.join --railway.topology.repair --offset.x 0 --offset.y 0
# Netconvert automáticamente transforma a UTM los datos de OSM, así que no hace falta la opción --proj.utm. Tan solo sumarle el offset que ha aplicado sumo para que el origen sea 0,0
        netconvert --osm-files ${map} --output-file ${network} --osm.stop-output.length 20 --ptstop-output ${ptstops_out} --ptline-output ${ptlines_out} --railway.topology.output ${railway} --geometry.remove --roundabouts.guess --ramps.guess --junctions.join --tls.guess-signals --tls.discard-simple --tls.join --railway.topology.repair --heightmap.geotiff ${altimetria_shp} --offset.x 0 --offset.y 0



	# Roundabouts.guess = "adivinar" salidas en rotondas
fi	
# 3. Create additional file for vehicle types and characteristics.
# Set a vehicle type distribution to include several vehicle types and save it in a file
echo "<additional>
    <vTypeDistribution id='typedist1'>
      	<vType id='type1' color='254,164,17' emissionClass='HBEFA3/PC_G_EU5' accel='1.8' length='9' maxSpeed='15' vClass='passenger'/>      	
	<vType id='type2' color='127,167,250'  emissionClass='HBEFA3/PC_G_EU5' accel='1.8' length='5' maxSpeed='15' vClass='motorcycle'/>
    </vTypeDistribution>
</additional>" > $add_file


# 3.2 Create public transport schedules. This creates both public transport trips and routes.
#python3 /home/alejandro/Documentos/sumoDocs/pythonScripts/ptlines2flows.py -n ${network} -s ${ptstops_out} -l ${ptlines_out} -o ${ptroutes} -p 300 --min-stops 1 --use-osm-routes --verbose
echo "Running ptlines2flows.py ..."
python3 $SUMO_HOME/tools/ptlines2flows.py -n ${network} -s ${ptstops_out} -l ${ptlines_out} -o ${ptroutes} -p 300 --min-stops 1 --use-osm-routes --verbose

# 4. Create non-public-transport trips (trip=begin/end points) with additional file where were defined the vehicle types to represent.
# There are more options of creating SUMO trips. For example, by reading origin/destination matrices (O/D matrices) with od2trips.py instead of randomtrips.py
echo "Running randomTrips.py ..."
python3 $SUMO_HOME/tools/randomTrips.py -n ${network} -e ${end_time} --trip-attributes='type="typedist1"' --additional-file ${add_file} --period 0.5  --edge-permission passenger -o ${trips}
#python3 $SUMO_HOME/tools/randomTrips.py -n ${network} -e ${end_time} --trip-attributes='type="typedist1"' --additional-file ${add_file} --period ((${end_time}-${begin_time})/${nveh})  --edge-permission passenger -o ${trips}


# 5. Create routes from non-public-transport trips.
if [ $dfrouter -eq 0 ]; then
	echo "Running duarouter..."
	duarouter -n ${network} --route-files ${trips} --additional-files ${add_file} -o ${routes} --ignore-errors
else
	echo "Running dfrouter..."
	sbatch run_dfrouter.sh 
fi

# 6. Create SUMO simulation configuration file
if [ $dfrouter -eq 0 ]; then
    echo "<?xml version='1.0' encoding='iso-8859-1'?>

    <configuration xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xsi:noNamespaceSchemaLocation='http://sumo.dlr.de/xsd/duarouterConfiguration.xsd'>

        <input>
            <net-file value='${network}'/>
            <route-files value='${ptroutes},${routes}'/>
	    <additional-files value='${ptstops_out}'/>
        </input>

        <output>
        	<emission-output value='${emi_out}'/>
	    <fcd-output value='${fcd_out}'/>
	    <fcd-output.distance value='true'/>
	    <fcd-output.acceleration value='true'/>
	    <tripinfo-output value='${trip_out}'/>
        </output>

        <time>
            <begin value='${begin_time}'/>
            <end value='${end_time}'/>
        </time>
     
        <processing>
            <time-to-teleport value='${teleport}'/>
        </processing>

    </configuration>" > $conf_file

else
echo "<?xml version='1.0' encoding='iso-8859-1'?>

    <configuration xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xsi:noNamespaceSchemaLocation='http://sumo.dlr.de/xsd/duarouterConfiguration.xsd'>

        <input>
            <net-file value='${network}'/>
            <route-files value='${ptroutes}'/>
	    <additional-files value='${ptstops_out},${routes},${emitters_file}'/>
        </input>

        <output>
        	<emission-output value='${emi_out}'/>
	    <fcd-output value='${fcd_out}'/>
	    <fcd-output.distance value='true'/>
	    <fcd-output.acceleration value='true'/>
	    <tripinfo-output value='${trip_out}'/>
        </output>

        <time>
            <begin value='${begin_time}'/>
            <end value='${end_time}'/>
        </time>
     
        <processing>
            <time-to-teleport value='${teleport}'/>
        </processing>

    </configuration>" > $conf_file
fi

# Extract a fcd output file in order to have data of each vehicle at each timestep
# Several type of outputs: 
# 1. Floating Car Data (--fcd-output) contains location, speed, vehicle angle (not slope), position, lane, etc. for every vehicle in the network at every time step
# 2. Raw vehicle positions dump (--netstate-dump): Contains every edges and lanes, each vehicle positions and speeds, for each sim. step
# 3. Emission output (--emission.output): Amount of pollutant gases, electricity and noise emitted by the vehicle in the actual simulation step
# 4. Full output (--full.output) contains every information in the network.

#sumo -c $conf_file --fcd-output $fcd_out --emission-output $emi_out --device.emissions.period "20.0" 

#sumo --net-file ${network} --additional-files ${routes},${emitters_file}
sumo -c $conf_file --ignore-route-errors
echo " \n sumo done"

# Open the configuration file in sumo-gui to launch the simulation
#sumo-gui $conf_file

# traceExporter.py converts fcd-output file to different file formats: OMNET, Shawn, ns2/ns3, PHEM
python3 $SUMO_HOME/tools/traceExporter.py --fcd-input $fcd_out --omnet-output fcd_to_omnet.xml #--kml-output fcd_to_kml.xml

# Transforming xml files to regular csv files
echo "Transforming xml output files to csv..."
python3 $SUMO_HOME/tools/xml/xml2csv.py ${emi_out} --output ${emi_csv}
python3 $SUMO_HOME/tools/xml/xml2csv.py ${trip_out} --output ${trip_csv}
python3 $SUMO_HOME/tools/xml/xml2csv.py $fcd_out
echo "done"

# Completar  con más pasos (por ejemplo emisiones)
# Siguiente problema: Cómo operar con ficheros de emisiones de SUMO (suma sobre todos los timestep, plotear valores en mapa, etc.)ç
