import numpy as np
import pandas as pd
import sumolib
import traci


def find_between( s, first, last ):
    try:
        start = s.index( first ) + len( first )
        end = s.index( last, start )
        return s[start:end]
    except ValueError:
        return ""

def find_between_r( s, first, last ):
    try:
        start = s.rindex( first ) + len( first )
        end = s.rindex( last, start )
        return s[start:end]
    except ValueError:
        return ""


# Read detector positions file
path = '/home/alejandro/Documentos/sumoDocs/DatosAforos/'
detectors_file = pd.read_csv(path+'UbicacionEstacionesPermanentesSentidos.csv',header=0,sep=';',decimal=',')
print(detectors_file)
detectors_file['Estación']=detectors_file['Estación'].apply(lambda x: '{0:0>2}'.format(x))
detectors_file['Id_sentido'] = np.zeros(len(detectors_file),str)
for i in range(len(detectors_file)):
    if detectors_file['Sentido'][i] == 1:
        detectors_file['Id_sentido'][i] = 'ES'+str(detectors_file['Estación'][i])+'-'+'a'
    else:
        detectors_file['Id_sentido'][i] = 'ES'+str(detectors_file['Estación'][i])+'-'+'b'


detectors_file.columns=(["ID","ESTACION","LATITUD","LONGITUD","SENTIDO","ORIENTACION","ID_SENTIDO"])
#detectors_file["LATITUD"].astype(float)

detectors_file['Edge'] = np.zeros(len(detectors_file),str)
detectors_file['Lane'] = np.zeros(len(detectors_file),str)
detectors_file['LanePos'] = np.zeros(len(detectors_file),str)
print(detectors_file)

detectors_file = detectors_file.dropna()
detectors_file.index = np.arange(0,len(detectors_file),1)

print(detectors_file)

# Locate nearby edges from detectors
simupath = '/home/alejandro/Documentos/sumoDocs/simulaciones/CAM_sinoffset/'
net = sumolib.net.readNet(simupath+'sumonetwork.net.xml')

radius = 10000
for i in range(len(detectors_file)): 
    x, y = net.convertLonLat2XY(detectors_file.LONGITUD[i], detectors_file.LATITUD[i], rawUTM=True)
    edges = net.getNeighboringEdges(x, y, radius)
    lane = net.getNeighboringLanes(x, y, radius)
    # lanePos, dist = sumolib.geomhelper.polygonOffsetAndDistanceToPoint((x,y), lane.getShape())

    # pick the closest edge
    if len(edges) > 0:
        # distancesAndEdges = sorted([(dist, edge) for edge, dist in edges])
        closestEdge, dist = min(edges, key = lambda t: t[1])
        closestLane, distLane = min(lane, key = lambda t: t[1])
        lanePos, dist = sumolib.geomhelper.polygonOffsetAndDistanceToPoint((x,y), closestLane.getShape())


    closestedge_id = str()
    closestedge_id=str(closestEdge)
    closestlane_id = str()
    closestlane_id=str(closestLane)
    
    print(dist,closestEdge)
    print(distLane,closestLane)

    if detectors_file['SENTIDO'][i]==1:
        detectors_file['Edge'][i] = find_between(closestedge_id, 'id="', '" f' )
        detectors_file['Lane'][i] = closestLane.getID()
        detectors_file['LanePos'][i] = lanePos
    elif detectors_file['SENTIDO'][i]==2:
        detectors_file['Edge'][i] = '-'+find_between(closestedge_id, 'id="', '" f' )
        detectors_file['Lane'][i] = closestLane.getID()
        detectors_file['LanePos'][i] = lanePos

# Extraigo como csv
detectors_file.to_csv("Detectors_file_with_netlocations.csv",index=False)
detectors_file_reduced = detectors_file.drop(columns=['ID','ESTACION','LATITUD','LONGITUD','SENTIDO','ORIENTACION','Edge'])
detectors_file_reduced.to_csv("Detectors_netlocations_Sentidos.csv",index=False)


# # Obtain also the position in lane nearest to the detector
# for i in range(len(detectors_file)): 
#     road, pos, index = traci.simulation.convertRoad(x,y)
#     detectors_file['Road'][i] = road
#     detectors_file['Pos'][i] = pos

    
